@@ -1059,13 +1059,13 @@ class Askdata(SignUp):
             response = s.put(url=url, json=settings, headers=headers)
             response.raise_for_status()
             # Execute sync
-            '''headers = {
+            headers = {
                 "Content-Type": "application/json",
                 "Authorization": "Bearer" + " " + self._token
             }
             dataset_url = self.base_url + '/smartdataset/datasets/' + dataset_id + '/sync'
             r = requests.post(url=dataset_url, headers=headers)
-            r.raise_for_status()'''
+            r.raise_for_status()
 
     def get_dataset_as_df(self, workspace:str, dataset_slug:str):
 
