@@ -29,6 +29,20 @@ consistent_datatypes = {"numeric": ["bit", "smallmoney", "money", "int",
                              "character", "character varying", "binary", "varbinary"]}
 
 
+def change_operator(operator):
+    if operator == "GOE":
+        operator = ">="
+    elif operator == "LOE":
+        operator = "<="
+    elif operator == "EQ":
+        operator = "=="
+    elif operator == "GT":
+        operator = ">"
+    elif operator == "LT":
+        operator = "<"
+    return operator
+
+
 class SQLFunction(Enum):
     MAX = auto()
     MIN = auto()
@@ -64,6 +78,46 @@ class Field:
     sqlExpression: Optional[str] = None
     sourceFormat: Optional[str] = None
 
+    def __repr__(self):
+        if self.aggregation is not None:
+            if isinstance(self.aggregation, SQLFunction) or isinstance(self.aggregation,
+                                                                        TimeDimensionGranularity):
+                field_with_agg = self.aggregation.name + " ( " + self.column + " )"
+            else:
+                field_with_agg = self.aggregation + " ( " + self.column + " )"
+        else:
+            field_with_agg = self.column
+        if self.alias is not None:
+            field_with_agg += " AS " + self.alias
+        return field_with_agg
+
+
+class MathOperator(Enum):
+    ADD = auto()
+    MUL = auto()
+    SUB = auto()
+    DIV = auto()
+
+
+@dataclass
+class ComplexMathField:
+    first_operand: Union[float, Field, str]
+    operator: Union[MathOperator, str]
+    second_operand: Union[float, Field, str]
+    percent: Optional[bool] = None
+    alias: Optional[str] = None
+
+    def __repr__(self):
+        if isinstance(self.operator, MathOperator):
+            repr = str(self.first_operand) + " " + self.operator.name + " " + str(self.second_operand)
+        else:
+            repr = str(self.first_operand) + " " + self.operator + " " + str(self.second_operand)
+        if self.percent is not None and self.percent:
+            repr += " %"
+        if self.alias is not None:
+            repr += " AS " + self.alias
+        return repr
+
 
 class SQLOperator(Enum):
     EQ = auto()
@@ -79,10 +133,60 @@ class SQLOperator(Enum):
     FOREACH = auto()
 
 
+@dataclass
+class ComplexConditionalField:
+    field: Field
+    operator: Union[SQLOperator, str]
+    value: Union[float, str]
+    else_clause: Union[bool, float, str]
+    then_clause: Optional[Union[bool, float, str]] = None
+    alias: Optional[str] = None
+
+    def __repr__(self):
+        if isinstance(self.operator, SQLOperator):
+            repr = str(self.field) + " " + self.operator.name + " " + str(self.value)
+        else:
+            repr = str(self.field) + " " + self.operator + " " + str(self.value)
+        repr += " THEN"
+        if self.then_clause is not None:
+            repr += " " + str(self.then_clause)
+        repr += " ELSE " + str(self.else_clause)
+        if self.alias is not None:
+            repr += " AS " + self.alias
+        return repr
+
+
 class PatternOperator(Enum):
     START = auto()
     END = auto()
     CONTAINS = auto()
+    REPLACE = auto()
+    FIND = auto()
+    LEN = auto()
+    REMOVE = auto()
+
+
+@dataclass
+class ComplexPatternField:
+    field: Field
+    operator: Union[PatternOperator, str]
+    source_pattern: Optional[str] = None
+    target_pattern: Optional[str] = None
+    alias: Optional[str] = None
+
+    def __repr__(self):
+        if isinstance(self.operator, PatternOperator):
+            repr = self.operator.name
+        else:
+            repr = self.operator
+        if self.source_pattern is not None:
+            repr += " " + self.source_pattern
+        if self.target_pattern is not None:
+            repr += " with " + self.target_pattern
+        repr += " " + str(self.field)
+        if self.alias is not None:
+            repr += " AS " + self.alias
+        return repr
 
 
 class BooleanOperator(Enum):
@@ -105,7 +209,7 @@ class TimeOperator(Enum):
 @dataclass
 class Condition:
     field: Field
-    operator: Optional[Union[SQLOperator, TimeOperator, str]] = None
+    operator: Optional[Union[SQLOperator, TimeOperator, PatternOperator, str]] = None
     value: Optional[List[Union[float, str]]] = None
     type: Optional[str] = None
     steps: Optional[str] = None
@@ -126,16 +230,7 @@ class Condition:
             operator = self.operator
         elif isinstance(self.operator, SQLOperator) or isinstance(self.operator, TimeOperator):
             operator = self.operator.name
-            if operator == "GOE":
-                operator = ">="
-            elif operator == "LOE":
-                operator = "<="
-            elif operator == "EQ":
-                operator = "=="
-            elif operator == "GT":
-                operator = ">"
-            elif operator == "LT":
-                operator = "<"
+            operator = change_operator(operator)
         else:
             operator = None
         field_with_agg = self.field.column
@@ -183,10 +278,29 @@ class Condition:
         return where_condition
 
 
+@dataclass
+class ComplexMathCondition:
+    field: ComplexMathField
+    operator: Union[SQLOperator, str]
+    value: List[Union[float, str]] = None
+
+    def __repr__(self):
+        repr = str(self.field)
+        if isinstance(self.operator, SQLOperator):
+            repr += " " + change_operator(self.operator.name)
+        else:
+            repr += " " + self.operator
+        values = []
+        for var in self.value:
+            values.append(str(var))
+        repr += " ( " + ", ".join(values) + " )"
+        return repr
+
+
 @dataclass
 class CompositeCondition:
     operator: Union[BooleanOperator, str]
-    conditions: List[Union[Condition, 'CompositeCondition']]
+    conditions: List[Union[Condition, ComplexMathCondition, 'CompositeCondition']]
 
     def __repr__(self):
         if isinstance(self.operator, BooleanOperator):
@@ -207,6 +321,11 @@ class Sorting:
     field: str
     order: Union[SQLSorting, str]
 
+    def __repr__(self):
+        if isinstance(self.order, SQLSorting):
+            return self.field + " " + self.order.name
+        return self.field + " " + self.order
+
 
 @dataclass
 class Component:
@@ -251,12 +370,12 @@ class GroupBy:
 
 @dataclass
 class Query:
-    fields: List[Field]
+    fields: List[Union[Field, ComplexMathField, ComplexConditionalField, ComplexPatternField]]
     pivot: Optional[List[Field]] = None
     join: Optional[List[Join]] = None
     id: Optional[str] = None
     datasets: Optional[List[From]] = None
-    where: Optional[List[Union[Condition, CompositeCondition]]] = None
+    where: Optional[List[Union[Condition, ComplexMathCondition, CompositeCondition]]] = None
     groupBy: Optional[List[GroupBy]] = None
     orderBy: Optional[List[Sorting]] = None
     limit: Optional[Union[int, str]] = None
@@ -265,27 +384,11 @@ class Query:
     def to_sql(self, dataset: str = None):
         sql = "SELECT {} FROM {}"
 
-        fields_with_agg = []
-        for field in self.fields:
-            column = field.column
-            if field.aggregation is not None:
-                if isinstance(field.aggregation, SQLFunction) or isinstance(field.aggregation,
-                                                                            TimeDimensionGranularity):
-                    field_with_agg = field.aggregation.name + " ( " + column + " )"
-                else:
-                    field_with_agg = field.aggregation + " ( " + column + " )"
-            else:
-                field_with_agg = column
-            if field.alias is not None:
-                field_with_agg += " AS " + field.alias
-            fields_with_agg.append(field_with_agg)
-
+        fields_with_agg = [str(field) for field in self.fields]
         formatted_fields = ", ".join(fields_with_agg)
 
-        froms_array = []
         if self.datasets is not None:
-            for f in self.datasets:
-                froms_array.append(f.dataset)
+            froms_array = [d.dataset for d in self.datasets]
             table = ", ".join(froms_array)
         elif dataset is not None:
             table = dataset
@@ -293,27 +396,18 @@ class Query:
             table = "{{dataset.A}}"
         sql = sql.format(formatted_fields, table)
 
-        where_conditions = []
         if self.where is not None:
             sql_where = " WHERE {}"
 
-            for condition in self.where:
-                where_condition = str(condition)
-                where_conditions.append(where_condition)
+            where_conditions = [str(condition) for condition in self.where]
             if where_conditions:
                 formatted_where_conditions = " AND ".join(where_conditions)
                 sql_where = sql_where.format(formatted_where_conditions)
                 sql += sql_where
 
-        sorting_conditions = []
         if self.orderBy is not None:
             sql_orderby = " ORDER BY {}"
-
-            for sorting in self.orderBy:
-                sort_order = sorting.order.name
-                sort_condition = sorting.field + " " + sort_order
-                sorting_conditions.append(sort_condition)
-
+            sorting_conditions = [str(sorting) for sorting in self.orderBy]
             formatted_sorting = ", ".join(sorting_conditions)
             sql_orderby = sql_orderby.format(formatted_sorting)
             sql += sql_orderby
